package de.androidnewcomer.myhighscoreserver;

import java.io.IOException;
import java.util.List;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.datanucleus.util.StringUtils;

import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.DatastoreServiceFactory;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.FetchOptions;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;
import com.google.appengine.api.datastore.Query;
import com.google.appengine.api.datastore.Query.FilterOperator;

@SuppressWarnings("serial")
public class MyHighscoreServerServlet extends HttpServlet {
	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		
		String game = req.getParameter("game");
		String name = req.getParameter("name");
		String pointsStr = req.getParameter("points");
		long points=0;
		if(pointsStr!=null) {
			points = Long.parseLong(pointsStr);
			if(points>10000000L) {
				resp.getWriter().println("score too high");
				return; // no dumb manipulated scores allowed 
			}
		}
		String maxStr = req.getParameter("max");
		int max = 10;
		if(maxStr!=null) {
			max = Integer.parseInt(maxStr);
		}
		
		if(points>0 && !StringUtils.isEmpty(name) && !StringUtils.isEmpty(game)) {
			// fing best score for game and name
			long previousPoints = findHighscore(game,name);
			if(points>previousPoints)
				addHighscore(game, name, points);
		}
		
		if(!StringUtils.isEmpty(game))
			returnHighscores(resp, game, max);
		else
			resp.getWriter().println("no game specified");
		
	}

	private long findHighscore(String game, String name) {
		DatastoreService datastore = DatastoreServiceFactory.getDatastoreService();
		Key gameKey = KeyFactory.createKey("game", game);
		Query query = new Query("highscore", gameKey).setFilter(new Query.FilterPredicate("name",FilterOperator.EQUAL,name));
		query.addSort("points", Query.SortDirection.DESCENDING);
		List<Entity> highscores = datastore.prepare(query).asList(FetchOptions.Builder.withLimit(1));
		if(!highscores.isEmpty()) {
			return (Long)highscores.get(0).getProperty("points");
		}
		return 0;
	}

	private void returnHighscores(HttpServletResponse resp, String game, int max) throws IOException {
		DatastoreService datastore = DatastoreServiceFactory.getDatastoreService();
		Key gameKey = KeyFactory.createKey("game", game);
		Query query = new Query("highscore", gameKey);
		query.addSort("points", Query.SortDirection.DESCENDING);
		List<Entity> highscores = datastore.prepare(query).asList(FetchOptions.Builder.withLimit(max));
		
		for(Entity e : highscores) {
			resp.getWriter().println(e.getProperty("name") + "," + e.getProperty("points"));
		}
	}

	private void addHighscore(String game, String name, long points) {
		DatastoreService datastore = DatastoreServiceFactory.getDatastoreService();
		Key gameKey = KeyFactory.createKey("game", game);
		Entity highscore = new Entity("highscore", gameKey);
		highscore.setProperty("name", name);
		highscore.setProperty("points", points);
		datastore.put(highscore);
		
	}
}
